#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import QUB
from QUBB import *
from Dialogs import *
from qubtree import Node
import CurveFitting

# implements old CurveFitting module api

class ExpoFitter:
    def __init__(self):
        self.state = Node('ExpoFit')
        self.paramsNode = self.state['Params']
        param = self.paramsNode['Param']
        param['Amp'].data = 1.0
        param['Tau'].data = 1.0
        param['Area'].data= 0.0
        self.params = [param]
        self.data = Node('FitData')
        self.data['X'].data = 1.0
        self.data['Y'].data = 1.0
        self.__n = 1
        self.algo = CurveFitting.MARQUARDT

    def setNumComponents(self, n):
        while n < len(self.params):
            self.paramsNode.remove( self.params.pop() )
        while n > len(self.params):
            newparam = self.params[0].clone()
            self.paramsNode.append( newparam )
            self.params.append( newparam )

    def autoFill(self):
        pass

    def getNumComponents(self):
        return len(self.params)

    def useData(self, dataset):
        self.data = dataset
        self.__n = min(dataset['X'].data.count, dataset['Y'].data.count)
        self.autoFill()
        
    def calcY(self, comp_ix, x, nextX=0.0):
        compute = self.state['Compute']
        if compute.data.count < 2:
            compute.data = [x, nextX]
        else:
            compute.data[0] = x
            compute.data[1] = nextX

        QUB.FitExpo_Eval(self.state, self.params[comp_ix])
        return compute.data[1]

    def getEqn(self):
        return 'Amp * exp( - x / Tau )'
        
    def promptOptions(self):
        chosenalgo = ''
        if self.algo == CurveFitting.MARQUARDT:
            chosenalgo = 'Marquardt'
        elif self.algo == CurveFitting.SIMPLEX:
            chosenalgo = 'Simplex'
        elif self.algo == CurveFitting.BFGS:
            chosenalgo = 'BFGS'
        else:
            chosenalgo = 'Simulated Annealing'
            
        options = QUB.Dialog('Curve fitting options',
                             ['OK', 'Cancel'],
                             [ DialogRadioGroup('algo', 'Algorithm:',
                                           ["Marquardt", "Simplex", "BFGS", "Simulated Annealing"],
                                           chosenalgo) ])
        if options['button'] == 'OK':
            algo = options['algo']
            if algo == 'Marquardt':
                self.algo = CurveFitting.MARQUARDT
            elif algo == 'Simplex':
                self.algo = CurveFitting.SIMPLEX
            elif algo == 'BFGS':
                self.algo = CurveFitting.BFGS
            else: # simulated annealing
                self.algo = CurveFitting.SIMANNEAL

    def getCurves(self, curves):
        return CurveFitting.RebuildCurves(self, curves)

    def fit(self, maxiter, precision=0.01):
        iters = self.state['MaxIter']
        iters.data = [maxiter, 0]
        self.state['Precision'].data = precision
        self.state['algo'].data = self.algo

        QUB.FitExpo_Fit(self.state, self.data)

        areasum = 0.0
        for param in self.params:
            areasum = areasum + param['Amp'].data[0] * param['Tau'].data[0]
        for param in self.params:
            param['Area'].data[0] = (param['Amp'].data[0] * param['Tau'].data[0]) / areasum

        try:
            err = self.state['Err'].data[0]
            if err == -1:
                QUB.Report('Fit error: singular hessian matrix')
            elif err == -2:
                QUB.Report("Fit error: too high Marquardt's parameter")
            elif err == -3:
                QUB.Report("Fit error: non-convergence")
        except:
            pass

CurveFitting.fitters['Exponential'] = ExpoFitter


class NegExpoFitter:
    def __init__(self):
        self.state = Node('NegExpoFit')
        self.paramsNode = self.state['Params']
        param = self.paramsNode['Param']
        param['Amp'].data = 1.0
        param['Tau'].data = 1.0
        self.params = [param]
        self.data = Node('FitData')
        self.data['X'].data = 1.0
        self.data['Y'].data = 1.0
        self.__n = 1
        self.algo = CurveFitting.MARQUARDT

    def setNumComponents(self, n):
        while n < len(self.params):
            self.paramsNode.remove( self.params.pop() )
        while n > len(self.params):
            newparam = self.params[0].clone()
            self.paramsNode.append( newparam )
            self.params.append( newparam )

    def autoFill(self):
        pass

    def getNumComponents(self):
        return len(self.params)

    def useData(self, dataset):
        self.data = dataset
        self.__n = min(dataset['X'].data.count, dataset['Y'].data.count)
        self.autoFill()
        
    def calcY(self, comp_ix, x, nextX=0.0):
        compute = self.state['Compute']
        if compute.data.count < 2:
            compute.data = [x, nextX]
        else:
            compute.data[0] = x
            compute.data[1] = nextX

        QUB.FitNegExpo_Eval(self.state, self.params[comp_ix])
        return compute.data[1]

    def getEqn(self):
        return 'Amp * (1 - exp( - x / Tau ))'
        
    def promptOptions(self):
        chosenalgo = ''
        if self.algo == CurveFitting.MARQUARDT:
            chosenalgo = 'Marquardt'
        elif self.algo == CurveFitting.SIMPLEX:
            chosenalgo = 'Simplex'
        elif self.algo == CurveFitting.BFGS:
            chosenalgo = 'BFGS'
        else:
            chosenalgo = 'Simulated Annealing'
            
        options = QUB.Dialog('Curve fitting options',
                             ['OK', 'Cancel'],
                             [ DialogRadioGroup('algo', 'Algorithm:',
                                           ["Marquardt", "Simplex", "BFGS", "Simulated Annealing"],
                                                chosenalgo) ])
        if options['button'] == 'OK':
            algo = options['algo']
            if algo == 'Marquardt':
                self.algo = CurveFitting.MARQUARDT
            elif algo == 'Simplex':
                self.algo = CurveFitting.SIMPLEX
            elif algo == 'BFGS':
                self.algo = CurveFitting.BFGS
            else: # simulated annealing
                self.algo = CurveFitting.SIMANNEAL

    def getCurves(self, curves):
        return CurveFitting.RebuildCurves(self, curves)

    def fit(self, maxiter, precision=0.01):
        iters = self.state['MaxIter']
        iters.data = [maxiter, 0]
        self.state['Precision'].data = precision
        self.state['algo'].data = self.algo

        QUB.FitNegExpo_Fit(self.state, self.data)

        try:
            err = self.state['Err'].data[0]
            if err == -1:
                QUB.Report('Fit error: singular hessian matrix')
            elif err == -2:
                QUB.Report("Fit error: too high Marquardt's parameter")
            elif err == -3:
                QUB.Report("Fit error: non-convergence")
        except:
            pass

CurveFitting.fitters['Neg Exponential'] = NegExpoFitter
